# CouchDB Wrapper
This module provides a simple wrapper class for couchdb requests.

## Installation

Using npm:
```
$ npm install --save couchdb-wrapper
```

Using bower:
```
$ bower install --save couchdb-wrapper
```

Using Script-Tag: *
```HTML
<script src="https://raw.githubusercontent.com/MCStreetguy/couchdb-wrapper/master/dist/couchdb-wrapper.min.js" charset="utf-8"></script>
```

Using short URL Script-Tag: *
```HTML
<script src="https://goo.gl/1YrYHF" charset="utf-8"></script>
```

_(* functionality is not granted!)_

## Usage

> This is currently missing any documentation about the babel-compiled module.
> Explanations below refer to the Node.js module provided in this package.

Require the module:
```JavaScript
import CouchDB from 'couchdb-wrapper';
```

Create a new instance:
```JavaScript
var db = new CouchDB({
  host: 'your database server',
  port: '5984 by default',
  username: 'login username',
  password: 'login password'
});
```
All options have to be set, otherwise an error will be thrown.
`username` and `password` can be empty strings if you don't need authentication on your CouchDB server.

### Command Reference

All functions are available as synchronous and asynchronous versions.
If the callback parameter is provided and a function, the call will be made asynchronously. Otherwise the request is sent synchronously.

**Please notice!** Synchronous requests block the execution of code which creates "freezing" on the screen and an unresponsive user experience. [Source (MDN)](https://developer.mozilla.org/en-US/docs/Web/API/XMLHttpRequest/Synchronous_and_Asynchronous_Requests)  
**But** since this leads straightly into "callbacks of doom" _(endless nesting of callback functions)_ a little hint: Firing the first function _(allDbs is always a good start for this)_ asynchronously splits the execution from the main process. Therefore you may use synchronous calls within a callback without freezing.

The following object scheme is provided to the callback or returned by the function, depending on the asynchrony.


|        Key | Description                                                                                                                               |
|-----------:|-------------------------------------------------------------------------------------------------------------------------------------------|
|   *status* | `'done'` if the request succeeded, `'pending'` if the request still has to be processed by the server or `'error'` if the request failed. |
|      *msg* | Contains a readable status message or an empty string if unnecessary.                                                                     |
| *response* | The response data or `undefined`.                                                                                                         |

---

#### **.allDbs(*[callback]*)**
Recieve all accessible databases stored on the server.

---

#### **.getDb(identifier,*[callback]*)**
Recieve a database object from the server.

`identifier`: The name of the database.

---

#### **.createDb(identifier,*[callback]*)**
Create a new database on the server.

`identifier`: The name of the database.

---

#### **.getDocs(identifier,*[callback]*)**
Recieve all documents within the given database.

`identifier`: The name of the database

---

#### **.postDoc(identifier,data,*[callback]*)**
Store an object in the given database.

`identifier`: The name of the database  
`data`: The object that should be stored in the database. If it contains `_id` the document will be stored under that ID, otherwise a new ID is generated and returned in the response.

---

#### **.getDoc(identifier,docIdentifier,*[callback]*)**
Store an object in the given database.

`identifier`: The name of the database  
`docIdentifier`: The uid of the document

---

#### **.deleteDoc(identifier,docIdentifier,revision,*[callback]*)**
Store an object in the given database.

`identifier`: The name of the database  
`docIdentifier`: The uid of the document  
`revision`: The latest revision id of the document

---

#### **.updateDoc(identifier,docIdentifier,newdata,*[callback]*)**
Store an object in the given database.

`identifier`: The name of the database  
`docIdentifier`: The uid of the document  
`newdata`: The new data to be written  
(`newdata.rev`: must contain the latest revision id)
